import React, { useState, useEffect } from 'react';
import { withRouter, matchPath } from 'react-router';
import { useSwipeable } from 'react-swipeable';
import SwipeableViews from 'react-swipeable-views';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var isarray = Array.isArray || function (arr) {
  return Object.prototype.toString.call(arr) == '[object Array]';
};

/**
 * Expose `pathToRegexp`.
 */
var pathToRegexp_1 = pathToRegexp;
var parse_1 = parse;
var compile_1 = compile;
var tokensToFunction_1 = tokensToFunction;
var tokensToRegExp_1 = tokensToRegExp;

/**
 * The main path matching regexp utility.
 *
 * @type {RegExp}
 */
var PATH_REGEXP = new RegExp([
  // Match escaped characters that would otherwise appear in future matches.
  // This allows the user to escape special characters that won't transform.
  '(\\\\.)',
  // Match Express-style parameters and un-named parameters with a prefix
  // and optional suffixes. Matches appear as:
  //
  // "/:test(\\d+)?" => ["/", "test", "\d+", undefined, "?", undefined]
  // "/route(\\d+)"  => [undefined, undefined, undefined, "\d+", undefined, undefined]
  // "/*"            => ["/", undefined, undefined, undefined, undefined, "*"]
  '([\\/.])?(?:(?:\\:(\\w+)(?:\\(((?:\\\\.|[^\\\\()])+)\\))?|\\(((?:\\\\.|[^\\\\()])+)\\))([+*?])?|(\\*))'
].join('|'), 'g');

/**
 * Parse a string for the raw tokens.
 *
 * @param  {string}  str
 * @param  {Object=} options
 * @return {!Array}
 */
function parse (str, options) {
  var tokens = [];
  var key = 0;
  var index = 0;
  var path = '';
  var defaultDelimiter = options && options.delimiter || '/';
  var res;

  while ((res = PATH_REGEXP.exec(str)) != null) {
    var m = res[0];
    var escaped = res[1];
    var offset = res.index;
    path += str.slice(index, offset);
    index = offset + m.length;

    // Ignore already escaped sequences.
    if (escaped) {
      path += escaped[1];
      continue
    }

    var next = str[index];
    var prefix = res[2];
    var name = res[3];
    var capture = res[4];
    var group = res[5];
    var modifier = res[6];
    var asterisk = res[7];

    // Push the current path onto the tokens.
    if (path) {
      tokens.push(path);
      path = '';
    }

    var partial = prefix != null && next != null && next !== prefix;
    var repeat = modifier === '+' || modifier === '*';
    var optional = modifier === '?' || modifier === '*';
    var delimiter = res[2] || defaultDelimiter;
    var pattern = capture || group;

    tokens.push({
      name: name || key++,
      prefix: prefix || '',
      delimiter: delimiter,
      optional: optional,
      repeat: repeat,
      partial: partial,
      asterisk: !!asterisk,
      pattern: pattern ? escapeGroup(pattern) : (asterisk ? '.*' : '[^' + escapeString(delimiter) + ']+?')
    });
  }

  // Match any characters still remaining.
  if (index < str.length) {
    path += str.substr(index);
  }

  // If the path exists, push it onto the end.
  if (path) {
    tokens.push(path);
  }

  return tokens
}

/**
 * Compile a string to a template function for the path.
 *
 * @param  {string}             str
 * @param  {Object=}            options
 * @return {!function(Object=, Object=)}
 */
function compile (str, options) {
  return tokensToFunction(parse(str, options), options)
}

/**
 * Prettier encoding of URI path segments.
 *
 * @param  {string}
 * @return {string}
 */
function encodeURIComponentPretty (str) {
  return encodeURI(str).replace(/[\/?#]/g, function (c) {
    return '%' + c.charCodeAt(0).toString(16).toUpperCase()
  })
}

/**
 * Encode the asterisk parameter. Similar to `pretty`, but allows slashes.
 *
 * @param  {string}
 * @return {string}
 */
function encodeAsterisk (str) {
  return encodeURI(str).replace(/[?#]/g, function (c) {
    return '%' + c.charCodeAt(0).toString(16).toUpperCase()
  })
}

/**
 * Expose a method for transforming tokens into the path function.
 */
function tokensToFunction (tokens, options) {
  // Compile all the tokens into regexps.
  var matches = new Array(tokens.length);

  // Compile all the patterns before compilation.
  for (var i = 0; i < tokens.length; i++) {
    if (typeof tokens[i] === 'object') {
      matches[i] = new RegExp('^(?:' + tokens[i].pattern + ')$', flags(options));
    }
  }

  return function (obj, opts) {
    var path = '';
    var data = obj || {};
    var options = opts || {};
    var encode = options.pretty ? encodeURIComponentPretty : encodeURIComponent;

    for (var i = 0; i < tokens.length; i++) {
      var token = tokens[i];

      if (typeof token === 'string') {
        path += token;

        continue
      }

      var value = data[token.name];
      var segment;

      if (value == null) {
        if (token.optional) {
          // Prepend partial segment prefixes.
          if (token.partial) {
            path += token.prefix;
          }

          continue
        } else {
          throw new TypeError('Expected "' + token.name + '" to be defined')
        }
      }

      if (isarray(value)) {
        if (!token.repeat) {
          throw new TypeError('Expected "' + token.name + '" to not repeat, but received `' + JSON.stringify(value) + '`')
        }

        if (value.length === 0) {
          if (token.optional) {
            continue
          } else {
            throw new TypeError('Expected "' + token.name + '" to not be empty')
          }
        }

        for (var j = 0; j < value.length; j++) {
          segment = encode(value[j]);

          if (!matches[i].test(segment)) {
            throw new TypeError('Expected all "' + token.name + '" to match "' + token.pattern + '", but received `' + JSON.stringify(segment) + '`')
          }

          path += (j === 0 ? token.prefix : token.delimiter) + segment;
        }

        continue
      }

      segment = token.asterisk ? encodeAsterisk(value) : encode(value);

      if (!matches[i].test(segment)) {
        throw new TypeError('Expected "' + token.name + '" to match "' + token.pattern + '", but received "' + segment + '"')
      }

      path += token.prefix + segment;
    }

    return path
  }
}

/**
 * Escape a regular expression string.
 *
 * @param  {string} str
 * @return {string}
 */
function escapeString (str) {
  return str.replace(/([.+*?=^!:${}()[\]|\/\\])/g, '\\$1')
}

/**
 * Escape the capturing group by escaping special characters and meaning.
 *
 * @param  {string} group
 * @return {string}
 */
function escapeGroup (group) {
  return group.replace(/([=!:$\/()])/g, '\\$1')
}

/**
 * Attach the keys as a property of the regexp.
 *
 * @param  {!RegExp} re
 * @param  {Array}   keys
 * @return {!RegExp}
 */
function attachKeys (re, keys) {
  re.keys = keys;
  return re
}

/**
 * Get the flags for a regexp from the options.
 *
 * @param  {Object} options
 * @return {string}
 */
function flags (options) {
  return options && options.sensitive ? '' : 'i'
}

/**
 * Pull out keys from a regexp.
 *
 * @param  {!RegExp} path
 * @param  {!Array}  keys
 * @return {!RegExp}
 */
function regexpToRegexp (path, keys) {
  // Use a negative lookahead to match only capturing groups.
  var groups = path.source.match(/\((?!\?)/g);

  if (groups) {
    for (var i = 0; i < groups.length; i++) {
      keys.push({
        name: i,
        prefix: null,
        delimiter: null,
        optional: false,
        repeat: false,
        partial: false,
        asterisk: false,
        pattern: null
      });
    }
  }

  return attachKeys(path, keys)
}

/**
 * Transform an array into a regexp.
 *
 * @param  {!Array}  path
 * @param  {Array}   keys
 * @param  {!Object} options
 * @return {!RegExp}
 */
function arrayToRegexp (path, keys, options) {
  var parts = [];

  for (var i = 0; i < path.length; i++) {
    parts.push(pathToRegexp(path[i], keys, options).source);
  }

  var regexp = new RegExp('(?:' + parts.join('|') + ')', flags(options));

  return attachKeys(regexp, keys)
}

/**
 * Create a path regexp from string input.
 *
 * @param  {string}  path
 * @param  {!Array}  keys
 * @param  {!Object} options
 * @return {!RegExp}
 */
function stringToRegexp (path, keys, options) {
  return tokensToRegExp(parse(path, options), keys, options)
}

/**
 * Expose a function for taking tokens and returning a RegExp.
 *
 * @param  {!Array}          tokens
 * @param  {(Array|Object)=} keys
 * @param  {Object=}         options
 * @return {!RegExp}
 */
function tokensToRegExp (tokens, keys, options) {
  if (!isarray(keys)) {
    options = /** @type {!Object} */ (keys || options);
    keys = [];
  }

  options = options || {};

  var strict = options.strict;
  var end = options.end !== false;
  var route = '';

  // Iterate over the tokens and create our regexp string.
  for (var i = 0; i < tokens.length; i++) {
    var token = tokens[i];

    if (typeof token === 'string') {
      route += escapeString(token);
    } else {
      var prefix = escapeString(token.prefix);
      var capture = '(?:' + token.pattern + ')';

      keys.push(token);

      if (token.repeat) {
        capture += '(?:' + prefix + capture + ')*';
      }

      if (token.optional) {
        if (!token.partial) {
          capture = '(?:' + prefix + '(' + capture + '))?';
        } else {
          capture = prefix + '(' + capture + ')?';
        }
      } else {
        capture = prefix + '(' + capture + ')';
      }

      route += capture;
    }
  }

  var delimiter = escapeString(options.delimiter || '/');
  var endsWithDelimiter = route.slice(-delimiter.length) === delimiter;

  // In non-strict mode we allow a slash at the end of match. If the path to
  // match already ends with a slash, we remove it for consistency. The slash
  // is valid at the end of a path match, not in the middle. This is important
  // in non-ending mode, where "/test/" shouldn't match "/test//route".
  if (!strict) {
    route = (endsWithDelimiter ? route.slice(0, -delimiter.length) : route) + '(?:' + delimiter + '(?=$))?';
  }

  if (end) {
    route += '$';
  } else {
    // In non-ending mode, we need the capturing groups to match as much as
    // possible by using a positive lookahead to the end or next path segment.
    route += strict && endsWithDelimiter ? '' : '(?=' + delimiter + '|$)';
  }

  return attachKeys(new RegExp('^' + route, flags(options)), keys)
}

/**
 * Normalize the given path string, returning a regular expression.
 *
 * An empty array can be passed in for the keys, which will hold the
 * placeholder key descriptions. For example, using `/user/:id`, `keys` will
 * contain `[{ name: 'id', delimiter: '/', optional: false, repeat: false }]`.
 *
 * @param  {(string|RegExp|Array)} path
 * @param  {(Array|Object)=}       keys
 * @param  {Object=}               options
 * @return {!RegExp}
 */
function pathToRegexp (path, keys, options) {
  if (!isarray(keys)) {
    options = /** @type {!Object} */ (keys || options);
    keys = [];
  }

  options = options || {};

  if (path instanceof RegExp) {
    return regexpToRegexp(path, /** @type {!Array} */ (keys))
  }

  if (isarray(path)) {
    return arrayToRegexp(/** @type {!Array} */ (path), /** @type {!Array} */ (keys), options)
  }

  return stringToRegexp(/** @type {string} */ (path), /** @type {!Array} */ (keys), options)
}
pathToRegexp_1.parse = parse_1;
pathToRegexp_1.compile = compile_1;
pathToRegexp_1.tokensToFunction = tokensToFunction_1;
pathToRegexp_1.tokensToRegExp = tokensToRegExp_1;

var patternCache = {};
var cacheLimit = 10000;
var cacheCount = 0;

var compileGenerator = function compileGenerator(pattern) {
  var cacheKey = pattern;
  var cache = patternCache[cacheKey] || (patternCache[cacheKey] = {});
  if (cache[pattern]) return cache[pattern];
  var compiledGenerator = pathToRegexp_1.compile(pattern);

  if (cacheCount < cacheLimit) {
    cache[pattern] = compiledGenerator;
    cacheCount++;
  }

  return compiledGenerator;
};

var generatePath = function generatePath(pattern, params) {
  if (pattern === void 0) {
    pattern = '/';
  }

  if (params === void 0) {
    params = {};
  }

  if (pattern === '/') {
    return pattern;
  }

  var generator = compileGenerator(pattern);
  return generator(params);
};

var RouterCarousel = function RouterCarousel(props) {
  var _useState = useState([]),
      urls = _useState[0],
      changeUrls = _useState[1];

  var _useState2 = useState(0),
      slideIndex = _useState2[0],
      changeSlideIndex = _useState2[1];

  var _useState3 = useState(false),
      routeHas = _useState3[0],
      changeRouteHas = _useState3[1];

  var _useState4 = useState(false),
      swipeleft = _useState4[0],
      toggleSwipeLeft = _useState4[1];

  var _useState5 = useState(false),
      swiperight = _useState5[0],
      toggleSwipeRight = _useState5[1];

  var _useState6 = useState(false),
      swipeall = _useState6[0],
      toggleSwipeAll = _useState6[1];

  var children = props.children,
      index = props.index,
      location = props.location,
      history = props.history,
      replace = props.replace,
      sliderMode = props.sliderMode,
      swipeLeftClassName = props.swipeLeftClassName,
      swipeRightClassName = props.swipeRightClassName,
      fallbackRoute = props.fallbackRoute;
  var nullElement = /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", null));
  var renderableRoutes = [];

  if (children !== null && children !== void 0 && children.length) {
    children.forEach(function () {
      renderableRoutes.push(nullElement);
    });
  } else {
    renderableRoutes.push(nullElement);
  }

  if (!sliderMode) {
    var slide = React.Children.toArray(children).find(function (element) {
      var _element$props;

      return (element === null || element === void 0 ? void 0 : (_element$props = element.props) === null || _element$props === void 0 ? void 0 : _element$props.path) === location.pathname;
    });
    renderableRoutes[slideIndex] = slide || nullElement;
  }

  var renderSlides = function renderSlides() {
    return renderableRoutes.map(function (element) {
      if (element !== null && element !== void 0 && element.props) {
        var _element$props3;

        var _element$props2 = element.props,
            path = _element$props2.path,
            component = _element$props2.component,
            render = _element$props2.render,
            _children = _element$props2.children;
        var _props = {
          location: location,
          history: history
        };
        var match = matchPath(location.pathname, element === null || element === void 0 ? void 0 : element.props);
        match = matchPath(generatePath(path, element === null || element === void 0 ? void 0 : (_element$props3 = element.props) === null || _element$props3 === void 0 ? void 0 : _element$props3.defaultParams), element === null || element === void 0 ? void 0 : element.props);
        _props.match = match;
        _props.key = path;
        return component ? React.createElement(component, _props) : render ? render(_props) : _children;
      }
    });
  };

  var triggerOnChangeIndex = function triggerOnChangeIndex() {
    React.Children.forEach(children, function (element, i) {
      var _element$props4 = element.props,
          pathProp = _element$props4.path,
          exact = _element$props4.exact,
          strict = _element$props4.strict,
          from = _element$props4.from;
      var path = pathProp || from;

      if (matchPath(history.location.pathname, {
        path: path,
        exact: exact,
        strict: strict
      })) {
        var _extends2;

        changeUrls(_extends({}, urls, (_extends2 = {}, _extends2[path] = history.location.pathname, _extends2)));
      }
    });
  };

  var createSlideUrl = function createSlideUrl(i) {
    var _React$Children$toArr = React.Children.toArray(children)[i].props,
        path = _React$Children$toArr.path,
        defaultParams = _React$Children$toArr.defaultParams;
    var url;

    if (path.includes(':')) {
      if (path in urls) {
        url = urls[path];
      } else {
        var _extends3;

        url = generatePath(path, defaultParams);
        changeUrls({
          urls: _extends({}, urls, (_extends3 = {}, _extends3[path] = url, _extends3))
        });
      }
    } else {
      url = path;
    }

    historyGoTo(url);
  };

  var handleIndexChange = function handleIndexChange(i) {
    if (!sliderMode) {
      createSlideUrl(i);
    }

    changeSlideIndex(i);
  };

  var historyGoTo = function historyGoTo(path) {
    return replace ? history.replace(path) : history.push(path);
  };

  var slideLeft = function slideLeft(activate) {
    if (activate) {
      var prevSlide = slideIndex > 0 ? slideIndex - 1 : 0;
      changeSlideIndex(prevSlide);

      if (!sliderMode) {
        createSlideUrl(prevSlide);
      }

      return true;
    }

    return false;
  };

  var slideRight = function slideRight(activate) {
    if (activate) {
      var numberOfSlides = !sliderMode ? renderableRoutes.length - 1 : children.length - 1;
      var nextSlide = slideIndex < numberOfSlides ? slideIndex + 1 : numberOfSlides;
      changeSlideIndex(nextSlide);

      if (!sliderMode) {
        createSlideUrl(nextSlide);
      }

      return true;
    }

    return false;
  };

  var handlerLeftSwipe = useSwipeable({
    onSwipedRight: function onSwipedRight() {
      return slideLeft(swipeleft);
    },
    preventDefaultTouchmoveEvent: true,
    trackMouse: true
  });
  var handlerRightSwipe = useSwipeable({
    onSwipedLeft: function onSwipedLeft() {
      return slideRight(swiperight);
    },
    preventDefaultTouchmoveEvent: true,
    trackMouse: true
  });

  var updateLocationPath = function updateLocationPath() {
    var match;
    React.Children.forEach(children, function (element, i) {
      var _element$props5 = element.props,
          pathProp = _element$props5.path,
          exact = _element$props5.exact,
          strict = _element$props5.strict,
          from = _element$props5.from;
      var path = pathProp || from;
      match = matchPath(location.pathname, {
        path: path,
        exact: exact,
        strict: strict
      });

      if (match) {
        changeSlideIndex(i);
      }
    });
  };

  var checkerSwipeOnSlide = function checkerSwipeOnSlide() {
    var _mode$slideIndex, _mode$slideIndex$prop, _mode$slideIndex2, _mode$slideIndex2$pro, _mode$slideIndex3, _mode$slideIndex3$pro, _mode$slideIndex4, _mode$slideIndex4$pro;

    var mode = !sliderMode ? renderableRoutes : children;

    if ((_mode$slideIndex = mode[slideIndex]) !== null && _mode$slideIndex !== void 0 && (_mode$slideIndex$prop = _mode$slideIndex.props) !== null && _mode$slideIndex$prop !== void 0 && _mode$slideIndex$prop.swipeleft) {
      toggleSwipeLeft(true);
    } else {
      toggleSwipeLeft(false);
    }

    if ((_mode$slideIndex2 = mode[slideIndex]) !== null && _mode$slideIndex2 !== void 0 && (_mode$slideIndex2$pro = _mode$slideIndex2.props) !== null && _mode$slideIndex2$pro !== void 0 && _mode$slideIndex2$pro.swiperight) {
      toggleSwipeRight(true);
    } else {
      toggleSwipeRight(false);
    }

    if ((_mode$slideIndex3 = mode[slideIndex]) !== null && _mode$slideIndex3 !== void 0 && (_mode$slideIndex3$pro = _mode$slideIndex3.props) !== null && _mode$slideIndex3$pro !== void 0 && _mode$slideIndex3$pro.swiperight || (_mode$slideIndex4 = mode[slideIndex]) !== null && _mode$slideIndex4 !== void 0 && (_mode$slideIndex4$pro = _mode$slideIndex4.props) !== null && _mode$slideIndex4$pro !== void 0 && _mode$slideIndex4$pro.swipeleft) {
      toggleSwipeAll(true);
    } else {
      toggleSwipeAll(false);
    }
  };

  var swipeLeftButton = /*#__PURE__*/React.createElement("section", _extends({}, handlerLeftSwipe, {
    className: swipeLeftClassName
  }));
  var swipeRightButton = /*#__PURE__*/React.createElement("section", _extends({}, handlerRightSwipe, {
    className: swipeRightClassName
  }));
  useEffect(function () {
    if (!sliderMode) {
      triggerOnChangeIndex();
      updateLocationPath();
    }
  }, []);
  useEffect(function () {
    if (!sliderMode) {
      updateLocationPath();
      changeRouteHas(renderableRoutes.some(function (route) {
        var _route$props, _route$props$path, _route$props2;

        if (route !== null && route !== void 0 && (_route$props = route.props) !== null && _route$props !== void 0 && (_route$props$path = _route$props.path) !== null && _route$props$path !== void 0 && _route$props$path.includes(':')) {
          var paramKey = Object.keys(route.props.defaultParams)[0];
          return route.props.path.replace(':' + paramKey, route.props.defaultParams[paramKey]) === location.pathname;
        }

        return (route === null || route === void 0 ? void 0 : (_route$props2 = route.props) === null || _route$props2 === void 0 ? void 0 : _route$props2.path) === location.pathname;
      }));
    }
  }, [location.pathname]);
  useEffect(function () {
    if (index && sliderMode) {
      var mode = !sliderMode ? renderableRoutes : children;
      var max = index >= (mode === null || mode === void 0 ? void 0 : mode.length) || index;
      var result = max >= 1 ? max - 1 : 1;
      changeSlideIndex(result);
    }
  }, [index]);
  useEffect(function () {
    checkerSwipeOnSlide();
  });
  return /*#__PURE__*/React.createElement(React.Fragment, null, sliderMode && swipeleft && swipeLeftButton, swipeleft && routeHas && swipeLeftButton, !sliderMode && routeHas && /*#__PURE__*/React.createElement(SwipeableViews, {
    index: slideIndex,
    onChangeIndex: handleIndexChange,
    disabled: swipeall
  }, renderSlides()), !routeHas && fallbackRoute || null, sliderMode && /*#__PURE__*/React.createElement(SwipeableViews, {
    index: slideIndex,
    onChangeIndex: handleIndexChange,
    disabled: swipeall
  }, children), swiperight && routeHas && swipeRightButton, sliderMode && swiperight && swipeRightButton);
};

var index = withRouter(RouterCarousel);

export default index;
//# sourceMappingURL=index.modern.js.map
